package fileAttente;
import java.util.ArrayList;
import java.util.concurrent.LinkedBlockingQueue;

public class FileDAttente {
    ArrayList<Poste> postesOuverts;
    LinkedBlockingQueue<Client> clientsEnAttente;

    public FileDAttente(int nbClientsMaxDansLaQueue){
        postesOuverts = new ArrayList<>();
        clientsEnAttente = new LinkedBlockingQueue<>(nbClientsMaxDansLaQueue);
    }

    public FileDAttente(){
        this(20);
    }

    public void ouvrePoste(Poste c){
        c.devientDisponible();
        if (!postesOuverts.contains(c)){
            postesOuverts.add(c);
        }
    }

    public void clientArrive(Client c){
        if (clientsEnAttente.remainingCapacity()==0){
            System.out.println("Client refusé");
        }
        clientsEnAttente.add(c);
    }

    public void accueilleClient(Poste poste){
        if(!poste.isDisponible()){
            return;
        }
        Client c = clientsEnAttente.poll();
        if (c != null){
            poste.prendClient(c);
        }
    }

    public void accueilleClient(){
        Poste premierDispo = null;
        for(Poste p : postesOuverts){
            if(p.isDisponible()){
                premierDispo = p;
            }
        }
        if (premierDispo != null) {
            accueilleClient(premierDispo);
        }
    }

    public double estimationTempsAttente(){
        double res = 0;
        for (Client c : clientsEnAttente){
            res += c.tempsTraitementEstime();
        }
        return res/postesOuverts.size();
    }

    public static void main(String[] args) {
        Poste p1 = new Poste(1);
        Poste p2 = new Poste(2);
        Poste p3 = new Poste(3);
        Client c1 = new Client();
        Client c2 = new Client();
        Client c3 = new Client();
        Client c4 = new Client();
        Client c5 = new Client();

        FileDAttente queue = new FileDAttente();
        queue.ouvrePoste(p1);
        queue.ouvrePoste(p2);
        queue.accueilleClient();
        queue.clientArrive(c1);
        queue.clientArrive(c2);
        queue.clientArrive(c3);
        queue.clientArrive(c4);
        queue.ouvrePoste(p3);
        queue.accueilleClient();
        p1.devientDisponible();
        queue.accueilleClient();
        queue.clientArrive(c5);
        p2.devientDisponible();
        p1.devientDisponible();
        queue.accueilleClient();
        p3.devientDisponible();
        queue.accueilleClient();
    }
}
